import { Form, Modal, Select } from 'antd';
import TextArea from 'antd/lib/input/TextArea';
import { useEffect, useState } from 'react';
import styles from './index.less';
import type { IFormValue } from '@/pages/PromptWord';

interface IEditPromptModalProps {
  open: boolean;
  setOpen: (value: boolean) => void;
  types: string[];
  modalType: 'create' | 'edit' | '';
  handleEditPrompt: (value: IFormValue) => void;
  oldRow: IFormValue;
  loading?: boolean;
  handleCreatePrompt: (value: IFormValue) => void;
}

const EditPromptModal = (props: IEditPromptModalProps) => {
  const { open, setOpen, types, modalType, handleEditPrompt, loading, oldRow, handleCreatePrompt } =
    props;

  const [currentSelectValue, setCurrentSelectValue] = useState<string | null>('请选择');
  const [textAreaValue, setTextAreaValue] = useState('');

  useEffect(() => {
    if (modalType === 'create') {
      setTextAreaValue('');
      setCurrentSelectValue('');
    } else {
      setTextAreaValue(oldRow.content);
      setCurrentSelectValue(oldRow.type || '请选择');
    }
  }, [open]);

  const handleSavePrompt = async () => {
    if (modalType === 'create') {
      await handleCreatePrompt({
        type: currentSelectValue,
        content: textAreaValue,
      });
    }

    if (modalType === 'edit') {
      await handleEditPrompt({
        type: currentSelectValue,
        content: textAreaValue,
      });
    }

    setOpen(false);
  };

  const layout = {
    labelCol: { span: 3 },
  };

  return (
    <Modal
      open={open}
      centered
      confirmLoading={loading}
      wrapClassName={styles.modal}
      title={modalType === 'create' ? '新增提示用语' : '编辑提示用语'}
      okText="保存"
      onOk={handleSavePrompt}
      onCancel={() => {
        setCurrentSelectValue('请选择');
        setOpen(false);
      }}
      width={574}
      bodyStyle={{
        padding: '20px',
      }}
    >
      <Form labelAlign="right" {...layout}>
        <Form.Item label="类型:">
          <Select
            onChange={(value: string) => {
              setCurrentSelectValue(value);
            }}
            value={currentSelectValue}
          >
            {types.map((item) => (
              <Select.Option value={item} key={item}>
                {item}
              </Select.Option>
            ))}
          </Select>
        </Form.Item>
        <Form.Item label="提示词语">
          <TextArea
            style={{
              height: '200px',
            }}
            value={textAreaValue}
            onChange={(e) => {
              setTextAreaValue(e.target.value);
            }}
          />
        </Form.Item>
      </Form>
    </Modal>
  );
};

export default EditPromptModal;
